# Program:			harFindString.pl
#
$pgmdescription = "
Description: This script will perform the following functions:

             1. List Harvest projects, states, and packages.
";

# Setup:			 This perl script is designed to produce a list of all harvest  
#                    Projects, States, and Packages.
#
# Example Usage Syntax:
# harFindString -tns orcl -dbusr harvest -dbpw ******* -b broker -str "search string" -mask "*.cpp,*.h"
#               -usr harvestdmin -pw *******
#
# Pre-Conditions:    You must have system environment variable defined  
#                    HARFINDSTRING=c:\HarFindString
# Post-Conditions:
#					See printHelp subroutine at the bottom of the program.
#
######################################################################################
# Errors:
#					001: Wrong  number of variables passed. 
#					002: Could not find the value for <INI File Key> under 
#									the <INI File Section> section in the <INI File Name>
#					003: <Variable Name> system variable is not set.
#					004: Could not find the mail program 
#									"<Name of the Mail program>".
#					005: Could not find the value for the requested 
#									information. (Could not find the value for 
#									"pass" under the "Oracle_Read" section in 
#									the <INI File Name>).
#					006: <SYSTEM VARIABLE> variable is not set.
#
######################################################################################
# Version: 
# 1.0 - carde11 01/16/2016  Code optimized for Oracle and SCM R12.1 and above.
######################################################################################
# Reading the UDP home variable
$home = "$ENV{'HARFINDSTRING'}" || die "006 - HARFINDSTRING system variable not set.\n";
$udphome = $home;
$temphome = $udphome;

# Declaring Perl Modules contains common subroutines...
use File::Basename;
use Term::ReadKey;
use Getopt::Long;

# Declaring Variables
$versionnum = "1.0"; 	# Version number of the program which should be changed everytime
						 # this file is updated.

my $logfile = "$udphome\\harFindString.log";

#open(OH, ">$logfile") || exiterror ("Can't open $logfile for output");
#select OH; $| = 1;
open STDERR, ">&STDOUT";
select STDERR; $| = 1;
select STDOUT; $| = 1;

# Command line parsing-- declare vars, then do it
use vars qw($opt_help, $opt_tns $opt_dbusr $opt_dbpw $opt_broker, $opt_usr, $opt_pw, $opt_str $opt_mask);
GetOptions('help','tns=s','dbusr=s','dbpw=s', 'broker=s', 'usr=s', 'pw=s', 'str=s','mask=s') || exiterror ("Could not parse command line\n");

# Check if the user asked for version or help

# Version informtion requested
if ($ARGV[0] eq "-v" or $ARGV[0] eq "-V") 
#if ($opt_help eq "-v" or $opt_help eq "-V")
	{
	# Print the version number
	print "Program:	Harvest Find String within File Versions\nVersion Number: $versionnum\n";
	exit 0;
	} # End of the version information
	# Help is requested	
elsif ($ARGV[0] eq "-h"	or $ARGV[0] eq "-H") 
#elsif ($opt_help eq "-h"	or $opt_help eq "-H") 
	{
	# Call help function
	printHelp();
	exit 0;
	} # End of the help function
	# Default section is to get the email address and send the email	
else 
	{
	##################################################
	# Reading Harvest Variables 
	##################################################


	# Left-shift the @ARGV elements until we only have the rest of the arguments 
	# (the number of shifts must match the number of ARGV since package list is last argv(6))...
	# Remove argv(0)
	#shift;	


	# We need the following for SQLPLUS...
	if (! defined $opt_tns)
		{
		ReadMode('normal');
		print("Please enter Oracle TNS Service Name: ");
		$opt_tns = ReadLine(0);
		chomp $opt_tns;
		}

	if (! defined $opt_dbusr)
		{		
		ReadMode('normal');
		print("Please enter SCM schema owner DB name: ");
		$opt_dbusr = ReadLine(0);
		chomp $opt_dbusr;
		}
			
	if (! defined $opt_dbpw)
		{				
		ReadMode('noecho');
		print("Please enter schema owner password (does not echo): ");
		$opt_dbpw = ReadLine(0);
		chomp $opt_dbpw;
		}

	if (! defined $opt_broker)
		{
		ReadMode('normal');
		print("Please enter the Broker Name: ");
		$opt_broker = ReadLine(0);
		chomp $opt_broker;
		}

	if (! defined $opt_usr)
		{
		ReadMode('normal');
		print("Please enter Harvest admin user Name: ");
		$opt_usr = ReadLine(0);
		chomp $opt_usr;
		}

	if (! defined $opt_tns)
		{
		ReadMode('noecho');
		print("Please enter Harvest admin user password: ");
		$opt_pw = ReadLine(0);
		chomp $opt_pw;
		}

	if (! defined $opt_str)
		{		
		ReadMode('normal');
		print("Please enter the match string (use double-quotes if it has spaces): ");
		$opt_str = ReadLine(0);
		chomp $opt_str;
		}

	if (! defined $opt_mask)
		{		
		ReadMode('normal');
		print("Please enter the file mask (example \"*.cpp,*.h\"): ");
		$opt_mask = ReadLine(0);
		chomp $opt_mask;
		}

	print("\n\n");

	$oraservice = $opt_tns;			
	$orauser    = $opt_dbusr;			
	$orapass    = $opt_dbpw;
	$broker     = $opt_broker;
	$adminusr   = $opt_usr;
	$adminpass  = $opt_pw;
	$search     = $opt_str;
	$mask       = $opt_mask;
	
	# for debug purposes...
	#print "ARGV[0]=$oraservice\n";
	#print "ARGV[1]=$orauser\n";
	#print "ARGV[2]=$orapass\n";
	#print "ARGV[3]=$opt_broker\n";
	#print "ARGV[4]=$opt_usr\n";
	#print "ARGV[5]=$opt_pw\n";
	#print "ARGV[6]=$search\n";
	#print "ARGV[7]=$mask\n";
	
	#Note: 'undefined' statenames are ignored... 
	$sql = "SELECT ENVIRONMENTNAME||\'=\'||STATENAME||\'>\'||PACKAGENAME   
            FROM HARENVIRONMENT, HARSTATE, HARPACKAGE
            WHERE HARSTATE.STATEOBJID = HARPACKAGE.STATEOBJID AND HARSTATE.ENVOBJID = HARENVIRONMENT.ENVOBJID
               AND HARSTATE.STATENAME != 'undefined'
               AND HARENVIRONMENT.ENVISACTIVE = 'Y'
            ORDER BY ENVIRONMENTNAME, STATENAME, PACKAGENAME";

	#Run SQLPlus to create the Harvest Project=State>Package list .dat file...
	#print "run SQL=$sql\n\n";
	@notifylist = runSQLPlus($usr, "$oraservice","$orauser","$orapass","$temphome", $sql);

	# read in the SQL results...
	chomp(@notifylist);
	#print "NotifyList=@notifylist\n";

	# Exit out of the program if there is no list to notify
	if ($#notifylist < 0) 
		{
    	print "\nWarning!! No data available from .dat file.";
    	exit 0;
		}   

	# read in the SQL results...
	$line = "";              # Temp line variable
	$project = "";		   # Temp Project variable
	$state = "";			 # Temp State variable
	$package = "";           # Temp Package variable

	# Remove any old findstr.log file...
	unlink("findstr.log");	
	# make sure the output directory is created for the hco's...
	system("mkdir $home\\output");

	my $iteration = 0;
	    
	# Loop through for each user in the list
	foreach $line (@notifylist) 
		{
    	chomp($line);
		#my ($project, $state, $package) = split /[=]/, split /[>]/, $line;
		$project = substr($line,0,index($line,'='));
		$state = substr($line, index($line,'=')+1, (index($line,'>')) - (index($line,'=')+1)); 
		$package = substr($line,index($line,'>')+1);

        # for debug purposes...
		#print "project=$project    state=$state    package=$package\n";

		if ($project ne "")
			{
			# TODO - hco with [project] [state] [package] filemask
			# There may be more than one file mask separated by commas...
			my @mask = split(/,/, $mask);
			#print "@mask\n";
			
			# We need to perform an hco and searchstring for each file mask...
			foreach $msk (@mask)
				{
				# Perform the hco...
				#print "msk=$msk\n";
				$msg="hco -b $broker -en $project -st $state -p $package $msk\n"; print "$msg";
				$cmd="hco -b \"$broker\" -en \"$project\" -st \"$state\" -p \"$package\" -vp \"/\" -cp \"$home\\output\" -sy -op pc -r -nt -s \"$msk\" -usr $adminusr -pw $adminpass";
				#$cmd="hco -b \"$broker\" -en \"$project\" -st \"$state\" -vp \"/\" -cp \"$home\\output\" -sy -op pc -r -nt -s \"$msk\" -usr $adminusr -pw $adminpass";
				$rc = system($cmd); if ($rc != 0) { print("ERROR: $rc $cmd\n"); exit 1; }
				$iteration = $iteration +1;
				}
			}				
				
    	} # End of the for loop

	if ($iteration ne 0)
		{
		# Now spin through the file mask and do a findstr...
		my @mask = split(/,/, $mask);
		foreach $msk (@mask)
			{		
			# Perform the findstring...
			$msg="findstr $search $msk\n"; print "$msg";
			$cmd= "FINDSTR /I /S \"$search\" .\\output\\$msk >> findstr.log";
			$rc = system($cmd); if ($rc != 0) { print("String Not Found: mask=$msk\n"); }
			}
			
		# Remove the hco output (cleanup)...
		$msg="rmdir .\\output\n"; print "$msg";
		$cmd ="rmdir $home\\output /s /Q";
		$rc = system($cmd); if ($rc != 0) { print("ERROR: $rc $cmd\n"); }
		}
			
	print "\nDone.\n\n";
  
} # End of the main else statement


##################################################
# Subroutine Section 
##################################################

#########################################################################################
#	Function:   printHelp
#
#	Input:      None.
#
#	Returns:	None.
# 
#	Description:	
#			This subroutine will print a help message to the standard output.
#
#	Call:       printHelp();
#
#	Pre-Conditions: 
#
#			    - None
#########################################################################################

sub printHelp 
{
print "
Program:	harFindString
Version:	$versionnum

$pgmdescription

Setup:  This is a standalone Perl script that will create a list of all Harvest 
        Projects, States, and Packages in this order.

Usage: perl harFindString.pl

Options:
		-h    Help
		-v    Version Number
		-tns  TNSService name
        -usr  Schema Owner name
        -pw   Schema Owner password
        -str  Search string (use dbl quotes if spaces)
        -mask Filemask dbl quote enclosed
        
Pre-Conditions:	
		- HARFINDSTRING environment variable path should be set.
		- Requires SQLPlus.
		- Harvest 12.1 or above.

Post-Conditions:
		None.											
					
If you are having difficult to read the help message 
then try running harFindString -h | more
";

}

#########################################################################################
# Program:		   harcommon.pl
# Setup:			 This is a perl script that has most of the common perl functions that are used 
#					for Harvest UDP.  
#
# Errors:			
#					harCommon-001: Called in writeFile when a file could not be created.
#					harCommon-002: Called in inputINI when the INI file is not found.
#					harCommon-003: Called in inputINI when the INI file could not be opened.
#					harCommon-004: Called in runHSQL when CA_SCM_HOME system variable not found.
#					harCommon-005: Called in runHSQL when the temporary file could not be 
#									created.
#					harCommon-006: Called in runHSQL when the temporary file could not be 
#									opened for reading.
#					harCommon-007: Called in runHUSQL when CA_SCM_HOME system variable not found.
#					harCommon-008: Called in runHUSQL when the temporary file could not be 
#									created.
#					harCommon-009: Called in runHUSQL when the temporary file could not be 
#									opened for reading.
#					harCommon-010: Called in runSQL when the temporary file could not be 
#									created.
#					harCommon-011: Called in runSQL when the temporary file could not be opened 
#									for reading.
#					harCommon-012: Called in exitError when CA_SCM_HOME system variable not found.

#########################################################################################
# Versions: 1.0 - Vibhav Mehrotra - September 20th 2001
#           1.1   carde11  10/14/2015  Added new runHSQLDFO uses a .dfo file for connection.
#
#########################################################################################

# Declaring Perl Modules
use File::Basename;

#########################################################################################
#	Function:	writeFile
#
#	Input:		- User Name of the person running the program
#				- Full path to the File along with the name.
#				- Text that needs to be written to the file
#
#	Returns:	None.
# 
#	Description:	
#				Subroutine to write the passed text in the specified file. 
#
#	Call:		writeFile("User Name", "<Full Path and File Name>","<Text to write to the file>");
#
#	Pre-Conditions: 
#
#			- The path of the file should exist.
#########################################################################################
sub writeFile 
{
my ($user, $myfile, $mytext) = @_;

# Check if the folder where the file is to be created exist.
my $filepath = dirname($myfile);
mkdir($filepath, 777) if (! -d $filepath);

# Open the file in append mode
open (mytmpfile, ">>$myfile") || exitError($user,"harCommon-001: Could not open the file \"$myfile\" for append.\n");

# Print the passed text in the file
print mytmpfile "$mytext\n";

# Close the file
close(mytmpfile);

} 

##########################################################################################
#	Function:	exitError
#
#	Description:
#				Subroutine to print error and exit the program with exit code 1.
#
#	Input:		- User Name of the person running the program
#				- Text to print out to the users.
#
#	Returns:	None.
# 
#	Call:		exitError("User Name","<Error Text>");
#
#	Pre-Conditions: 
#
#				- NONE.
#########################################################################################
sub exitError 
{
# Read the passed Variables
my ($user, $errmsg) = @_;

##################################################
# Declaring all the required Files and directories
##################################################

$loglocation = "$home\\udplogs";

if (! -d "$loglocation") 
	{
	# Making the log directory
	mkdir("$loglocation",777);
	}

$udplog = $loglocation . "\\udperrors.log";

$execdate = scalar localtime();

$writemsg = "$execdate\t$user\t$errmsg";

# Write the Error to the log file
writeFile($user, $udplog, $writemsg);

# Print the error to the session log
print "\nERROR! $errmsg\nExiting.......\n";

# Exit
exit(1);
}

#########################################################################################
#	Function:	repSpace_Colon
#
#	Description:
#				Subroutine to substitute all the colons and spaces from a date to underscores (_).
#
#	Input:		Date 
#
#	Returns:	Reformatted Date
# 
#	Call:		repSpace_Colon("<Date Value>");
#
#	Pre-Conditions: 
#
#			- None.
#########################################################################################
sub repSpace_Colon 
{
my $dttime = "@_";

$dttime =~ s/ /_/g;
$dttime =~ s/:/_/g;

return $dttime;
}

#########################################################################################
#	Function:	getOS
#
#	Description:
#				Subroutine to find out which operating system we are running the perl script on.
#
#	Input:		None
#
#	Returns:	- All Windows Operating System will return "NT".
#				- All UNIX Operating System will return "unix".
#				- Any other Operating System will return "undefined". 
# 
#	Call:		getOS();
#
#	Pre-Conditions: 
#
#			- None.
#
#	Alternative to this function would be (Use the code below in the main program):
# 
#	if ($^O eq "MSWin32")
#		{ 	
#		}
#	else 
#		{
#		}	
#########################################################################################
sub getOS
{
    return ("NT")   if (system ("ver") == 0);
    return ("UNIX") if (system ("uname") == 0);
    return ("undefined");
}

#########################################################################################
#	Function:	replaceStr
#
#	Description:
#				Subroutine to replace any matching substring with replace string.
#
#	Input:	  - The find string
#				- The replace string
#				- The input string to scan
#
#	Returns:	Updated string
# 
#	Call:		replaceStr($find, $replace, $string);
#
#	Pre-Conditions: 
#
#			- None.
#
#########################################################################################
sub replaceStr
{
	my($find, $replace, $string) = @_;

	$pos = index($string, $find);
	
	while ( $pos > -1 ) 
		{
    	substr( $string, $pos, length( $find ), $replace );
		$pos = index( $string, $find, $pos + length( $replace ));
		}
		
	return $string;		
}

#########################################################################################
#	Function:	inputINI
#
#	Description:
#				Subroutine to read information from the specified INI files.
#
#	Input:	  - User Name of the person running the program
#				- INI File name along with it path.
#				- INI section (Case Sensitive)
#				- Section Key whose value is requested
#
#	Returns:	Value of the Key if found else (##@##)
# 
#	Call:		inputINI("<User Name>","<INI File Name>","<INI Section>","<Section Key>");
#
#	Pre-Conditions: 
#
#			- INI file name and the path should exist.
#
#	The main program should use the following code example to handle the subroutine
#
#		if ( $variable ne "(##@##)") {
#			....<Do Something>.....
#		 }
#		else {
#			....<Error Condition when a value is not found>......
#		}
##################################################
sub inputINI 
{
# Assigning the sections passed by the program
 my($user, $inifilename,$inisection,$inikey) = @_;

 $inisection = "[" . "$inisection" . "]";

# Declaring local variables
 my @inivalues = "";
 my $leftval, $rightval, $finalval, $tmpval = "";
 my $sectionreached, $valuefound = 0;

# Check to see if the specified file exist
 exitError($user, "inputINI-002: Could not find INI File: $inifilename.") if (! -e $inifilename); 

# Reading the file content into an array
 open (inifilein, "$inifilename") ||exitError($user, "inputINI-003: Could not open the file \"$inifilename\" for reading purposes.\n");
 @inivalues = <inifilein>;
 close(inifilein);

# Looping through to find our section and our key
 foreach $tmpval (@inivalues) 
	{
	# Removing the end line characters
	chomp($tmpval);

	# Find our section in the ini file
	if (($inisection eq $tmpval) and !$sectionreached) 
		{
		# We have reached our section and now we can start looking for our value
		$sectionreached = 1;
		} # End of the section test
		# If we reached our section then we can start looking for our value 
	elsif ($sectionreached and !$valuefound) 
		{
		# Split the values 	   
		($leftval, $rightval) = split("=",$tmpval);
		  
		# Check if we find the key
		if ($leftval =~ /$inikey/i) 
			{
			# We found the key so we can exit out of the sub routine with our value
			$valuefound = 1;
			$rightval    =~ s/ +$//;
			$rightval    =~ s/^ +//;
			return $rightval;
			} # End of the leftvalue comparision
		} # End of the value search
	} # End of the for loop

# If we reached this part of the subroutine then we did not find the code so we will exit with a 
# special character "(##@##)"
 return "(##@##)";

} # End of the inputINI subroutine

#########################################################################################
#	Function:	runHSQL
#
#	Description:
#				Subroutine to run only a "select" statement through the Harvest command line hsql
#
#	Input:		- User Name of the person running the program
#				- Harvest Broker Name
#				- Harvest ID
#				- Password of that ID
#				- Temporary Location where files can be created
#				- SQL Statement to be executed
#
#	Returns:		Result of the SQL Statement in an Array
#					Returns "Temporary Folder Does Not Exist" if the folder does not exist.
# 
#	Call:		runHSQL("<Broker>","<Harvest ID>","<Harvest Password>","<Temporary Location",
#						"<SQL Statement>");
#
#	Pre-Conditions: 
#
#			- The temporary location should exist.
#			- The Harvest ID and password should be valid in Harvest.
#########################################################################################
sub runHSQL 
{
 my($user, $hbroker, $hsqluser, $hsqlpass, $loglocation, $sql) = @_;

 my $hhome = "$ENV{'CA_SCM_HOME'}" || exitError($user, "harCommon-004: CA_SCM_HOME system variable not set.");

 my $hsqlsyntax = "$hhome\/hsql -b $hbroker -usr $hsqluser -pw $hsqlpass";

# Check if the location exist
 return "Temporary Folder Does Not Exist" if (! -d $runloc);

# Generating the temporaty file locations where the sql statements will be written.
 my($tmpfilesql) = $loglocation . "\/" . "hsql" . time() . ".sql";
 my($tmpfiledat) = $loglocation . "\/" . "hsql" . time() . ".dat";

# Check which Platform we are in and set the slashes accordingly
 if ($^O eq "MSWin32")
	{ 	
	$tmpfilesql =~ s/\//\\/g;
	$tmpfiledat =~ s/\//\\/g;		
	$hsqlsyntax =~ s/\//\\/g;
	}
 
# We will save the SQL statement in a file...
 open (mysql, ">$tmpfilesql") ||exitError($user, "harCommon-005: Could not open the file \"$tmpfilesql\" for writing purposes.\n");

 print mysql "$sql;";

# Close the file
 close(mysql);

##################################################
# Running the Harvest hsql to get the result we want.
# The HSQL will be run with the following options:
#
#	-t		- Tabular format
#	-nh		- No header
#	-f		- Input File
#	-o		- Output File
##################################################

 system "$hsqlsyntax -nh -t -f $tmpfilesql -o $tmpfiledat";

# Getting the result from the SQL Statement.
 open (RES,"<$tmpfiledat") || exitError($user, "harCommon-006: Could not open the file \"$tmpfiledat\" for read only purposes\n");

 @hsqlresult = <RES>;
 close(RES);
  
 if ($hsqlresult[0] eq "") 
	{
	shift(@hsqlresult);
	}

# In Harvest 5 the log file has three extralines of info so we delete those
 shift @hsqlresult;
 shift @hsqlresult;
 shift @hsqlresult;

 unlink($tmpfilesql);
 unlink($tmpfiledat);

 return (@hsqlresult);

} # End of runHSQL

#########################################################################################
#	Function:	runHSQLDFO  (hsql -eh <dfo file>)
#
#	Description:
#				Subroutine to run only a "select" statement through the Harvest command line hsql
#                using path to a .dfo file for hsql -eh parameter.
#
#	Input:      - User Name of the person running the program
#				- Harvest Broker Name
#				- Harvest ID
#				- Password of that ID
#				- Temporary Location where files can be created
#				- SQL Statement to be executed
#
#	Returns:		Result of the SQL Statement in an Array
#					Returns "Temporary Folder Does Not Exist" if the folder does not exist.
# 
#	Call: runHSQL("<Broker>","<DFO Filepath>","<Temporary Location", "<SQL Statement>");
#
#	Pre-Conditions: 
#
#			- The temporary location should exist.
#			- The Harvest ID and password should be valid in Harvest.
#########################################################################################
sub runHSQLDFO
{
 my($user, $hbroker, $hsqldfo, $runloc, $sql) = @_;

	# debug
	#	print "\nrunHSQLDFO: user=$user\n";
	#	print "runHSQLDFO: broker=$hbroker\n";
	#	print "runHSQLDFO: dfo=$hsqldfo\n";
	#	print "runHSQLDFO: runloc=$runloc\n";
	
 my $hhome = "$ENV{'CA_SCM_HOME'}" || exitError($user, "runHSQLDFO-004: CA_SCM_HOME variable is not set.");

 my $hsqlsyntax = "\"$hhome\/hsql\" -b $hbroker -eh \"$hsqldfo\"";

# Check if the location exist
 return "Temporary Folder Does Not Exist" if (! -d $runloc);

# Generating the temporaty file locations where the sql statements will be written.
 my($tmpfilesql) = $runloc . "\/" . "hsql" . time() . ".sql";
 my($tmpfiledat) = $runloc . "\/" . "hsql" . time() . ".dat";

# Check which Platform we are in and set the slashes accordingly
 if ($^O eq "MSWin32")
	{ 	
	$tmpfilesql =~ s/\//\\/g;
	$tmpfiledat =~ s/\//\\/g;		
	$hsqlsyntax =~ s/\//\\/g;
	}
 
 #	print "runHSQLDFO: tmpfilesql=$tmpfilesql\n";
 #	print "runHSQLDFO: tmpfiledat=$tmpfiledat\n";
 
# We will save the SQL statement in a file...
 open (mysql, ">$tmpfilesql") || exitError($user, "runHSQLDFO: 005 - Could not open the file \"$tmpfilesql\" for write.\n");

# write the sql to the file...
 print mysql "$sql;";

# Close the file.
 close(mysql);

##################################################
# Running the Harvest hsql to get the result we want.
# The HSQL will be run with the following options:
#
#	-t		- Tabular format
#	-nh		- No header
#	-f		- Input File
#	-o		- Output File
##################################################

 #print "Executing: $hsqlsyntax -nh -t -f $tmpfilesql -o $tmpfiledat";
 system "$hsqlsyntax -nh -t -f $tmpfilesql -o $tmpfiledat";

# Getting the result from the SQL Statement.
 open (RES,"<$tmpfiledat") || exitError($user, "runHSQLDFO: 006 - Could not open the file \"$tmpfiledat\" for read.\n");

 @hsqlresult = <RES>;

 #	print "\nrunHSQLDFO: hsqlresult=@hsqlresult\n";
 
 close(RES);
  
 if ($hsqlresult[0] eq "") 
	{
	shift(@hsqlresult);
	}
 else
	{	
	print "\nCreated: \"$tmpfiledat\"\n";
	}
	
# In Harvest 5 the log file has three extralines of info so we delete those.
# Commented out for Harvest 12.1 and above...
 #shift @hsqlresult;
 #print "hsqlresult3=@hsqlresult\n";
 #shift @hsqlresult;
 #print "hsqlresult4=@hsqlresult\n";
 #shift @hsqlresult;
 #print "hsqlresult5=@hsqlresult\n";

 #unlink deletes the files...
 unlink($tmpfilesql);
 #Do not delete the dat file...
 #unlink($tmpfiledat);

 return (@hsqlresult);

} # End of runHSQLDFO

#########################################################################################
#	Function:	runHUSQL
#
#	Description:
#				Subroutine to run any SQL statement through Harvest command line husql
#
#	Input:      - User Name of the person running the program
#				- Harvest Broker Name
#				- Harvest ID
#				- Password of that ID
#				- Temporary Location where files can be created
#				- SQL Statement to be executed
#
#	Returns:	Result of the SQL Statement in an Array
#				Returns "Temporary Folder Does Not Exist" if the folder does not exist.
# 
#	Call: runHUSQL("<Broker>","<Harvest ID>","<Harvest Password>","<Temporary Location", "<SQL Statement>");
#
#	Pre-Conditions: 
#
#			- The temporary location should exist.
#			- The Harvest ID and password should be valid in Harvest.
#########################################################################################
sub runHUSQL 
{
my($user, $hbroker, $hsqluser, $hsqlpass, $runloc ,$sql) = @_;

my $hhome = "$ENV{'CA_SCM_HOME'}" || exitError($user, "harCommon-007: CA_SCM_HOME system variable not set.");

my $husqlsyntax = "$hhome\/husql  -b $hbroker -usr $hsqluser -pw $hsqlpass ";

# Check if the location exist
return "Temporary Folder Does Not Exist" if (! -d $runloc);

# Generating the temporaty file locations where the sql statements will be written.
 my($tmpfilesql) = $runloc . "\/" . "hsql" . time() . ".sql";
 my($tmpfiledat) = $runloc . "\/" . "hsql" . time() . ".dat";

# Check which Platform we are in and set the slashes accordingly
if ($^O eq "MSWin32")
	{ 	
	$tmpfilesql =~ s/\//\\/g;
	$tmpfiledat =~ s/\//\\/g;		
	$husqlsyntax =~ s/\//\\/g;
	}

##################################################
# We will save the SQL statement in a file.
##################################################

open (mysql, ">$tmpfilesql") ||exitError($user, "harCommon-008: Could not open the file \"$tmpfilesql\" for writing purposes.\n");

print mysql "$sql";

# Close the file
close(mysql);


##################################################
# Running the Harvest husql to get the result we want.
# The HUSQL will be run with the following options:
#
#	-f	- Input file
#	-o	- Output file 
##################################################

system "$husqlsyntax -f $tmpfilesql -o $tmpfiledat";

# Getting the result from the SQL Statement.
open (RES,"<$tmpfiledat")||exitError($user, "harCommon-009: Could not open the file \"$tmpfiledat\" for read only purposes.\n");
  @hsqlresult = <RES>;
  close(RES);
  
  if ($hsqlresult[0] eq "") 
  	{
  	shift(@hsqlresult);
	  }

# In Harvest 5 the log file has three extralines of info so we delete those
shift @hsqlresult;
shift @hsqlresult;
shift @hsqlresult;

  unlink($tmpfilesql);
  unlink($tmpfiledat);

 return (@hsqlresult);

} # End of runHUSQL

#########################################################################################
#	Function:	runSQLPlus
#
#	Description:
#				Subroutine to run any SQL statement through SQLPlus
#
#	Input:      - User Name of the person running the program
#				- Oracle Service (Alais) Name
#				- Oracle ID
#				- Password of that ID
#				- Temporary Location where files can be created
#				- SQL Statement to be executed
#
#	Returns:	Results of the SQL Statement in an Array
#				Returns "Temporary Folder Does Not Exist" if the folder does not exist.
# 
#	Call:  runSQLPlus("<Service name>","<ID>","<Password>","<Temporary Location>", "<SQL Statement>")
#
#	Pre-Conditions: 
#
#			- The temporary location should exist.
#			- The Oracle ID and password should have the appropriate access in Oracle to run the 
#				passed SQL Statement.
#			- Oracle's Bin directory should be in the System's PATH.
#
#	CAUTION:
#
#			USE THIS SUBROUTINE WHERE YOU EXPECT THE RESULT OF THE SQL STATEMENT WILL BE 
#			SMALL. 
#########################################################################################
sub runSQLPlus
{
 my($user, $sqlservice, $dbuname, $dbpword, $runloc, $sql) = @_;
 my $sqlplus = "sqlplus -s"; 

#debug - print "sqlservice=$sqlservice\n";

# Check if the location exist
return "Temporary Folder Does Not Exist" if (! -d $runloc);

# Generating the temporary file locations where the sql statements will be written.
 my($tmpfilesql) = $runloc . "\/" . "hsql" . time() . ".sql";
 my($tmpfiledat) = $runloc . "\/" . "hsql" . time() . ".dat";

 # Fixing the slashes for the NT operating system
if ($^O eq "MSWin32") 
	{
	$tmpfilesql =~ s/\//\\/g;
	$tmpfiledat =~ s/\//\\/g;
	}  

#debug - print "tmpfilesql=$tmpfilesql\n";

  # Generating the SQL statements.
  open (SQL,">$tmpfilesql") || exitError($user, "SqlPlus-010: Could not open the file \"$tmpfilesql\" for write purposes.\n");
  print SQL "set pagesize 0;\n";
  print SQL "spool &1;\n";
  print SQL "set trimspool on;\n";
  print SQL "set heading off;\n";
  print SQL "set headsep off;\n";
  print SQL "set recsep off;\n";
  print SQL "set verify off;\n";
  print SQL "set feedback off;\n";
  print SQL "set termout off;\n";
  print SQL "set wrap off;\n";
  print SQL "set echo off;\n";
  print SQL "$sql;\n";
  print SQL "exit;\n";
   
  close(SQL);
  
# Running the SQL Statement.
# debug - print "$sqlplus $dbuname\/$dbpword\@$sqlservice \@$tmpfilesql $tmpfiledat\n\n";
  system "$sqlplus $dbuname\/$dbpword\@$sqlservice \@$tmpfilesql $tmpfiledat";

# Getting the result from the SQL Statement.
  open (RES,"<$tmpfiledat") || exitError($user, "SqlPlus-011: Could not open the file \"$tmpfiledat\" for read.\n");
  @QList = <RES>;
  close(RES);

  if ($QList[$#QList] =~ /Input truncated /) {pop @QList;}
  
  if ($QList[0] =~ /Input truncated /) {shift @QList;}
  
  if ($QList[0] eq "") {shift(@QList);}

  print "\nCreated: \"$tmpfiledat\"\n\n";

  # unlink will delete these files...
  unlink($tmpfilesql);
  unlink($tmpfiledat);

 return (@QList);
}

#########################################################################################
#	Function:	findStringArray
#
#	Description:
#				Subroutine to find the string in the passed array 
#
#	Input:		- String to search for in the array
#				- Array 
#
#	Returns:	0 - When the string is not found in the array
#				1 - Found the string in the array
# 
#	Call:		findStringArray("<String to Find>","<Array>")
#
#	Pre-Conditions: 
#
#			- None.
#########################################################################################

sub findStringArray
{
my ($mystring, @myarray) = @_;

my $tmp = "";

# Looping through each element to see if the string matches.
foreach $tmp (@myarray) 
	{
	return 1 if  ($mystring eq $tmp);
	}

return 0;

} # End of the subroutine    


########################################################
# Since this file will not run by itself I am including the true statement at the end:
1;






